@@ -1,10 +1,17 @@
-from typing import Type, Optional
-from mohou.trainer import TrainCache
-from mohou.model import AutoEncoderBase, LSTM
-from mohou.types import AngleVector, GripperState, TerminateFlag, MultiEpisodeChunk, get_all_concrete_leaftypes
-from mohou.embedder import IdenticalEmbedder
-from mohou.embedding_rule import EmbeddingRule
+from typing import Optional, Type
+
+from mohou.encoder import VectorIdenticalEncoder
+from mohou.encoding_rule import EncodingRule
+from mohou.model import LSTM, AutoEncoderBase
 from mohou.propagator import Propagator
+from mohou.trainer import TrainCache
+from mohou.types import (
+    AngleVector,
+    GripperState,
+    MultiEpisodeChunk,
+    TerminateFlag,
+    get_all_concrete_leaftypes,
+)
 
 
 class DefaultNotFoundError(Exception):
@@ -36,7 +43,7 @@ def auto_detect_autoencoder_type(project_name: Optional[str] = None) -> Type[Aut
     return t
 
 
-def create_default_embedding_rule(project_name: Optional[str] = None) -> EmbeddingRule:
+def create_default_encoding_rule(project_name: Optional[str] = None) -> EncodingRule:
 
     chunk = MultiEpisodeChunk.load(project_name)
     chunk_spec = chunk.spec
@@ -50,20 +57,22 @@ def create_default_embedding_rule(project_name: Optional[str] = None) -> Embeddi
 
     if tcache_autoencoder.best_model is None:
         raise DefaultNotFoundError("traincache was found but best model is not saved ")
-    image_embed_func = tcache_autoencoder.best_model.get_embedder()
-    av_idendical_func = IdenticalEmbedder(AngleVector, av_dim)
+    image_encoder = tcache_autoencoder.best_model.get_encoder()
+    av_idendical_encoder = VectorIdenticalEncoder(AngleVector, av_dim)
 
-    embedders = [image_embed_func, av_idendical_func]
+    encoders = [image_encoder, av_idendical_encoder]
 
     if GripperState in chunk_spec.type_shape_table:
-        gs_identital_func = IdenticalEmbedder(GripperState, chunk_spec.type_shape_table[GripperState][0])
-        embedders.append(gs_identital_func)
+        gs_identital_func = VectorIdenticalEncoder(
+            GripperState, chunk_spec.type_shape_table[GripperState][0]
+        )
+        encoders.append(gs_identital_func)
 
-    tf_identical_func = IdenticalEmbedder(TerminateFlag, 1)
-    embedders.append(tf_identical_func)
+    tf_identical_func = VectorIdenticalEncoder(TerminateFlag, 1)
+    encoders.append(tf_identical_func)
 
-    embed_rule = EmbeddingRule.from_embedders(embedders, chunk)
-    return embed_rule
+    encoding_rule = EncodingRule.from_encoders(encoders, chunk)
+    return encoding_rule
 
 
 def create_default_propagator(project_name: Optional[str] = None) -> Propagator:
@@ -72,7 +81,7 @@ def create_default_propagator(project_name: Optional[str] = None) -> Propagator:
     except Exception:
         raise DefaultNotFoundError("not TrainCache for lstm is found ")
 
-    embed_rule = create_default_embedding_rule(project_name)
+    encoding_rule = create_default_encoding_rule(project_name)
     assert tcach_lstm.best_model is not None
-    propagator = Propagator(tcach_lstm.best_model, embed_rule)
+    propagator = Propagator(tcach_lstm.best_model, encoding_rule)
     return propagator
