@@ -36,8 +36,8 @@ from mohou.image_randomizer import (
 )
 from mohou.setting import setting
 from mohou.utils import (
+    assert_equal_with_message,
     assert_isinstance_with_message,
-    assert_with_message,
     canvas_to_ndarray,
     get_all_concrete_leaftypes,
     split_sequence,
@@ -185,8 +185,8 @@ class PrimitiveImageBase(PrimitiveElementBase, ImageBase):
 
     def __init__(self, data: np.ndarray) -> None:
         super().__init__(data)
-        assert_with_message(self._data.ndim, 3, "image_dim")
-        assert_with_message(data.shape[2], self.channel(), "channel")
+        assert_equal_with_message(self._data.ndim, 3, "image_dim")
+        assert_equal_with_message(data.shape[2], self.channel(), "channel")
 
     @classmethod
     def channel(cls) -> int:
@@ -196,7 +196,7 @@ class PrimitiveImageBase(PrimitiveElementBase, ImageBase):
 class ColorImageBase(PrimitiveImageBase, Generic[ColorImageT]):
     def __init__(self, data: np.ndarray) -> None:
         super().__init__(data)
-        assert_with_message(self._data.dtype.type, np.uint8, "numpy type")
+        assert_equal_with_message(self._data.dtype.type, np.uint8, "numpy type")
 
     def to_tensor(self) -> torch.Tensor:
         return torchvision.transforms.ToTensor()(self._data).float()
@@ -279,7 +279,7 @@ class DepthImage(PrimitiveImageBase):
 
     def __init__(self, data: np.ndarray) -> None:
         super().__init__(data)
-        assert_with_message(
+        assert_equal_with_message(
             self._data.dtype.type, [np.float16, np.float32, np.float64], "numpy type"
         )
 
@@ -329,7 +329,7 @@ class CompositeImageBase(ImageBase):
 
         if check_size:
             for image in images:
-                assert_with_message(image.shape[:2], image_shape, "image w-h")
+                assert_equal_with_message(image.shape[:2], image_shape, "image w-h")
 
         for image, image_type in zip(images, self.image_types):
             assert_isinstance_with_message(image, image_type)
@@ -393,7 +393,7 @@ class ElementDict(Dict[Type[ElementBase], ElementBase]):
     def __init__(self, elems: Sequence[ElementBase]):
         for elem in elems:
             self[elem.__class__] = elem
-        assert_with_message(len(set(self.keys())), len(elems), "num of element")
+        assert_equal_with_message(len(set(self.keys())), len(elems), "num of element")
 
     def __getitem__(self, key: Type[ElementT]) -> ElementT:
         if issubclass(key, PrimitiveElementBase):
@@ -448,10 +448,6 @@ class ElementSequence(HasAList[ElementT], Generic[ElementT]):
         assert elem.shape == self.elem_shape
         self.elem_list.append(elem)
 
-    def get_partial(self, indices: List[int]) -> "ElementSequence[ElementT]":
-        elems = [self.elem_list[idx] for idx in indices]
-        return ElementSequence(elems)
-
 
 def create_composite_image_sequence(
     composite_image_type: Type[CompositeImageT],
@@ -617,7 +613,7 @@ class MultiEpisodeChunk(HasAList[EpisodeData], TypeShapeTableMixin):
 
         n_type_appeared = len(set_types)
         n_type_expected = len(data_list[0].types())
-        assert_with_message(n_type_appeared, n_type_expected, "num of element in chunk")
+        assert_equal_with_message(n_type_appeared, n_type_expected, "num of element in chunk")
 
         data_list_intact = []
         if with_intact_data:
@@ -719,23 +715,45 @@ class MultiEpisodeChunk(HasAList[EpisodeData], TypeShapeTableMixin):
         self.type_shape_table = other.type_shape_table
 
     def plot_vector_histories(
-        self, elem_type: Type[VectorBase], project_name: Optional[str] = None
+        self,
+        elem_type: Type[VectorBase],
+        project_name: Optional[str] = None,
+        hz: Optional[float] = None,
     ) -> None:
 
         n_vec_dim = self.spec.type_shape_table[elem_type][0]
 
         fig = plt.figure()
-        gs = fig.add_gridspec(n_vec_dim, hspace=0)
-        axs = gs.subplots(sharex=True, sharey=True)
+        fig, axs = plt.subplots(n_vec_dim)
 
         for i_dim, ax in enumerate(axs):
+
+            y_min, y_max = +np.inf, -np.inf  # will be updated in the following loop
+
             for data in self.data_list:
                 seq = data.get_sequence_by_type(AngleVector)
                 single_seq = np.array([e.numpy()[i_dim] for e in seq])
-                axs[i_dim].plot(single_seq, color="red", lw=0.5)
+                y_min = min(y_min, np.min(single_seq))
+                y_max = max(y_max, np.max(single_seq))
+
+                if hz is None:
+                    axs[i_dim].plot(single_seq, color="red", lw=0.5)
+                else:
+                    time_seq = [i / hz for i in range(len(single_seq))]
+                    axs[i_dim].plot(time_seq, single_seq, color="red", lw=0.5)
+
+            margin = 0.2
+            diff = y_max - y_min
+            axs[i_dim].set_ylim([y_min - diff * margin, y_max + diff * margin])
 
         for ax in axs:
             ax.grid()
+
+        if hz is None:
+            fig.supxlabel("data point number [-]")
+        else:
+            fig.supxlabel("time [s]")
+
         project_path = get_project_path(project_name)
         file_path = project_path / "seq-{}.png".format(elem_type.__name__)
         file_name = str(file_path)
